package UI;
/*** In The Name of Allah ***/

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.io.File;

/**
 * This class holds the state of the game and all of its elements.
 * This class also handles user inputs, which affect the game state.
 *
 * @author Seyed Mohammad Ghaffarian
 */
public class GameState {

    public int locX, locY, diam, rotate;
    public boolean gameOver;

    private boolean keyUP, keyDOWN, keyRIGHT, keyLEFT;
    private boolean mousePress;
    private int mouseX, mouseY;
    private KeyHandler keyHandler;
    private MouseHandler mouseHandler;
    private WallState walls;

    public GameState(WallState walls) {
        this.walls = walls;
        // Initialize the game state and all elements ...
        locX = 100;
        locY = 100;
        rotate = 4;

        diam = 32;
        gameOver = false;

        keyUP = false;
        keyDOWN = false;
        keyRIGHT = false;
        keyLEFT = false;

        mousePress = false;
        mouseX = 0;
        mouseY = 0;

        keyHandler = new KeyHandler();
        mouseHandler = new MouseHandler();
        keyHandler = new KeyHandler();
        mouseHandler = new MouseHandler();
    }

    /**
     * The method which updates the game state.
     */
    public void update() {
        //
        // Update the state of all game elements
        //  based on user input and elapsed time ...
        //

        int lastx = locX,lasty = locY,lastr = rotate;

        if (mousePress) {
            locY = mouseY - diam / 2;
            locX = mouseX - diam / 2;
        }
        if (keyUP) {
            rotate = rotate % 360;
            locY -= 8;
            if ( (rotate> 90 && rotate <180) || rotate== 90 )
                rotate +=10;
            else if ( (rotate >180 && rotate < 270) || rotate ==270 )
                rotate -=10;
            else
                rotate =180;
        }
        if (keyDOWN) {
            rotate = rotate % 360;
            locY += 8;
            if ( (rotate > 270 && rotate < 360) || rotate ==270 )
                rotate +=10;
            else if ( (rotate < 90 && rotate > 0) || rotate == 90 )
                rotate -=10;
            else
                rotate = 0;
        }
        if (keyLEFT ){
            rotate = rotate % 360;
            locX -= 8;
            if ( (rotate  > 0 && rotate < 90 )|| rotate == 0)
                rotate +=10;
            else if ( (rotate > 90 && rotate <180 )|| rotate == 180 )
                rotate -=10;
            else
                rotate = 90;
            }
        if (keyRIGHT ) {
            rotate = rotate % 360;
            locX += 8;
            if ((rotate > 180 && rotate < 270)|| rotate ==180 )
                rotate +=10;
            else if ( (rotate > 270 && rotate < 360) || rotate == 360)
                rotate -=10;
            else
                rotate =270;
        }

        if (walls.isWall(locX,locY)){
            locX = lastx;
            locY = lasty;
        }

        locX = Math.max(locX, 0);
        locX = Math.min(locX, MainFrame.GAME_WIDTH - diam);
        locY = Math.max(locY, 0);
        locY = Math.min(locY, MainFrame.GAME_HEIGHT - diam);

    }


    public KeyListener getKeyListener() {
        return keyHandler;
    }
    public MouseListener getMouseListener() {
        return mouseHandler;
    }
    public MouseMotionListener getMouseMotionListener() {
        return mouseHandler;
    }



    /**
     * The keyboard handler.
     */
    class KeyHandler extends KeyAdapter {

        @Override
        public void keyPressed(KeyEvent e) {

            switch (e.getKeyCode())
            {
                case KeyEvent.VK_UP:
                    keyUP = true;
                    break;
                case KeyEvent.VK_DOWN:
                    keyDOWN = true;
                    break;
                case KeyEvent.VK_LEFT:
                    keyLEFT = true;
                    break;
                case KeyEvent.VK_RIGHT:
                    keyRIGHT = true;
                    break;
                case KeyEvent.VK_ESCAPE:
                    gameOver = true;
                    break;
            }
        }

        @Override
        public void keyReleased(KeyEvent e) {
            switch (e.getKeyCode())
            {
                case KeyEvent.VK_UP:
                    keyUP = false;
                    break;
                case KeyEvent.VK_DOWN:
                    keyDOWN = false;
                    break;
                case KeyEvent.VK_LEFT:
                    keyLEFT = false;
                    break;
                case KeyEvent.VK_RIGHT:
                    keyRIGHT = false;
                    break;
            }
        }

    }

    /**
     * The mouse handler.
     */
    class MouseHandler extends MouseAdapter {

        @Override
        public void mousePressed(MouseEvent e) {
            mouseX = e.getX();
            mouseY = e.getY();
            mousePress = true;
        }

        @Override
        public void mouseReleased(MouseEvent e) {
            mousePress = false;
        }

        @Override
        public void mouseDragged(MouseEvent e) {
            mouseX = e.getX();
            mouseY = e.getY();
        }
    }
}

